//Apply an image to the destination (usually "screen") with offsets x and y
void applySurface( int x, int y, SDL_Surface* sourceImage, SDL_Surface* destination, SDL_Rect* clippingRect = NULL )
{
      //Rectangle to hold offsets
      SDL_Rect offsetRect;

      //Calculate offsets
      offsetRect.x = x;
      offsetRect.y = y;

      //Blit the image to the screen
      SDL_BlitSurface( sourceImage, clippingRect, destination, &offsetRect );
}

//Apply an image to the destination (usually "screen") with offsets x and y scaled to handle screen size
void applyScaledCenteredSurface( int x, int y, SDL_Surface* sourceImage, SDL_Surface* destination, SDL_Rect* clippingRect = NULL )
{
      //Holds offsets
      SDL_Rect offsetRect;

      //Get offsets
      offsetRect.x = scaleFactorX * x - sourceImage->w / 2;
      offsetRect.y = scaleFactorY * y - sourceImage->h / 2;

      //Blit
      SDL_BlitSurface( sourceImage, clippingRect, destination, &offsetRect );
}

void applyScaledCircularSurface( int x, int y, double angle, SDL_Surface* baseImage, SDL_Surface* sourceImage, SDL_Surface* destination, SDL_Rect* clippingRect = NULL )
{
      //Holds offsets
      SDL_Rect offsetRect;

      //Get offsets
      offsetRect.x = scaleFactorX * x + baseImage->w / 2 * cos( angle ) - sourceImage->w / 2 ;
      offsetRect.y = scaleFactorY * ( MAX_SCREEN_HEIGHT - y ) - baseImage->h / 2 * sin( angle ) - sourceImage->h / 2;

      //Blit
      SDL_BlitSurface( sourceImage, clippingRect, destination, &offsetRect );
}

void createScreen( int addScreenInfoEvent, int requestedWidth, int requestedHeight, int requestedBPP, int safeWidth = 640, int safeHeight = 480, int safeBPP = 8 )
{
      int realBPPNeeded = 0,
          finalBPP = 0,
          failed = false;

      if ( addScreenInfoEvent )
      {
            myStringstream.clear();
            myStringstream.str("");
            myStringstream << "Attempting to set up screen in ";
            if ( fullscreen )
                  myStringstream << "fullscreen";
            else
                  myStringstream << "windowed";
            myStringstream << " mode with " << requestedWidth << "x" << requestedHeight << " resolution and " << requestedBPP << " Bit color depth";

            addEvent( myStringstream.str().c_str(), 6000 );
      }

      if ( fullscreen )
      {
            realBPPNeeded = SDL_VideoModeOK( requestedWidth, requestedHeight, requestedBPP, SDL_FULLSCREEN );

            //If fullscreen screen cannot be created with desired parameters under any color depth, try windowed mode
            if ( realBPPNeeded == 0 )
            {
                  realBPPNeeded = SDL_VideoModeOK( requestedWidth, requestedHeight, requestedBPP, SDL_HWSURFACE );

                  //If mode is supported, use returned color depth (realBPPNeeded) to create windowed screen
                  if ( realBPPNeeded != 0 )
                  {
                        failed = true;
                        screen = SDL_SetVideoMode( requestedWidth, requestedHeight, realBPPNeeded, SDL_HWSURFACE );
                        SCREEN_WIDTH = requestedWidth;
                        SCREEN_HEIGHT = requestedHeight;
                        finalBPP = realBPPNeeded;
                        fullscreen = false;
                  }
                  //If mode won't work at all, use safe settings (previously used settings if passed as parameters, or simply defaults) to create a windowed screen
                  else
                  {
                        failed = true;
                        screen = SDL_SetVideoMode( safeWidth, safeHeight, safeBPP, SDL_HWSURFACE );
                        SCREEN_WIDTH = safeWidth;
                        SCREEN_HEIGHT = safeHeight;
                        finalBPP = safeBPP;
                  }
            }
            //Else if fullscreen mode supported, use returned color depth (realBPPNeeded) to create screen
            else
            {
                  screen = SDL_SetVideoMode( requestedWidth, requestedHeight, realBPPNeeded, SDL_FULLSCREEN );
                  SCREEN_WIDTH = requestedWidth;
                  SCREEN_HEIGHT = requestedHeight;
                  finalBPP = realBPPNeeded;
            }
      }
      else //If windowed mode requested
      {
            realBPPNeeded = SDL_VideoModeOK( requestedWidth, requestedHeight, requestedBPP, SDL_HWSURFACE );

            //If mode is supported, use returned color depth (realBPPNeeded) to create windowed screen
            if ( realBPPNeeded != 0 )
            {
                  screen = SDL_SetVideoMode( requestedWidth, requestedHeight, realBPPNeeded, SDL_HWSURFACE );
                  SCREEN_WIDTH = requestedWidth;
                  SCREEN_HEIGHT = requestedHeight;
                  finalBPP = realBPPNeeded;
            }
            //If mode won't work at all, use safe settings (previously used settings if passed as parameters, or simply defaults) to create a windowed screen
            else
            {
                  failed = true;
                  screen = SDL_SetVideoMode( safeWidth, safeHeight, safeBPP, SDL_HWSURFACE );
                  SCREEN_WIDTH = safeWidth;
                  SCREEN_HEIGHT = safeHeight;
                  finalBPP = safeBPP;
            }
      }

      //Set the new BPP to the one actually used to create the screen
      SCREEN_BPP = finalBPP;

      //If there was an error in setting up the screen
      if( screen == NULL )
            exit ( 2000 );

      if ( addScreenInfoEvent )
      {
            myStringstream.clear();
            myStringstream.str("");

            if ( failed == false && requestedBPP == SCREEN_BPP )
            {
                  myStringstream << "Change in video mode succeeded";
            }
            else if ( failed == false && requestedBPP != SCREEN_BPP )
            {
                  myStringstream << "Change in video mode succeeded, and BPP changed to " << SCREEN_BPP;
            }
            else
            {
                  myStringstream << "Attempt failed, falling back to ";
                  if ( fullscreen )
                        myStringstream << "fullscreen";
                  else
                        myStringstream << "windowed";
                  myStringstream << " mode in resolution " << SCREEN_WIDTH << "x" << SCREEN_HEIGHT << " with color depth " << SCREEN_BPP;
            }

            addEvent( myStringstream.str().c_str(), 6000 );
      }
}

bool initialize()
{
      //Initialize all standard SDL subsystems
      if( SDL_Init( SDL_INIT_EVERYTHING ) == -1 )
            return false;

      //Initialize SDL_ttf
      if( TTF_Init() == -1 )
            return false;

      //Initialize SDL_mixer
      if( Mix_OpenAudio( SAMPLE_RATE, MIX_DEFAULT_FORMAT, 2, SAMPLE_SIZE ) == -1 )
            return false;

      //Create the screen
      createScreen( false, SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP );

      //Set the window caption
      SDL_WM_SetCaption( "SDL Asteroids", NULL );

      //If everything initialized fine
      return true;
}

SDL_Surface *load_image( std::string filename, int makeTransparency, int red, int green, int blue )
{
      SDL_Surface *loadedImage = NULL;
      SDL_Surface *optimizedImage = NULL;

      loadedImage = IMG_Load( filename.c_str() );

      //If the image loaded
      if( loadedImage != NULL )
      {
            //Create an optimized image
            optimizedImage = SDL_DisplayFormat( loadedImage );

            //Free the old image
            SDL_FreeSurface( loadedImage );
      }

      //If the image was optimized just fine
      if( optimizedImage != NULL && makeTransparency )
      {
            //Map the color key (white)
            Uint32 colorkey = SDL_MapRGB( optimizedImage->format, red, green, blue );

            //Set all pixels of color R, G, B to be transparent
            SDL_SetColorKey( optimizedImage, SDL_SRCCOLORKEY, colorkey );
      }

      //Return the optimized image
      return optimizedImage;
}

SDL_Surface *loadScaledImage( std::string filepath,
                              int setColorKey,
                              int color1, int color2, int color3,
                              double scalefactorX, double scalefactorY,
                              int smooth,
                              int finalColorkey )
{
      SDL_Surface *returnMe = NULL,
                  *bigImage = NULL;

      bigImage = load_image( filepath, setColorKey, color1, color2, color3 );
      returnMe = zoomSurface( bigImage, scalefactorX, scalefactorY, smooth );
      SDL_FreeSurface( bigImage );

      if ( returnMe == NULL )
            quitGame( 110 );

      SDL_SetColorKey( returnMe, SDL_SRCCOLORKEY, finalColorkey );

      return returnMe;
}

//Load all images, properly scaled
void load_images()
{
      int index = 1,
          i;

      //Temporary SDL Surfaces
      SDL_Surface *testImage = NULL,
                  *testPreviewImage = NULL,
                  *testParticleImage = NULL,
                  *bigImage = NULL;

      //The temporary Ship Image object that will be used to store all the variously colored ship images
      shipImage testShipImage;

      //Load non-scaled images
      exaustParticleImage =      load_image( "data/images/particles/exaustParticle.png", true, 255, 255, 255 );
      explosionParticleImage =   load_image( "data/images/particles/explosionParticle.png", true, 255, 255, 255 );

      //Check non-scaled images to make sure they loaded right, exit if they didn't
      if ( exaustParticleImage == NULL ||
           explosionParticleImage == NULL )
            exit ( 100 );

      //Load scaled images
      background = loadScaledImage( "data/images/backgrounds/background.png", false, 0, 0, 0, SCREEN_WIDTH / 1600.0, SCREEN_HEIGHT / 1200.0, 1, 0 );

      if ( background != NULL )
      {
            //Display the background
            applySurface( 0, 0, background, screen );
      }
      if ( smallFont != NULL )
      {
            TTF_CloseFont( smallFont );
            smallFont = TTF_OpenFont( "data/fonts/LiberationSans-Bold.ttf", ( 35 * SCREEN_HEIGHT ) / 768 );
            text = TTF_RenderText_Solid( smallFont, "Loading files, please wait...", blackColour );
            applySurface( SCREEN_WIDTH / 2 - text->w / 2, SCREEN_HEIGHT / 2 - text->h / 2, text, screen );
            SDL_FreeSurface( text );
      }
      //Update the screen (actually display the new frame)
      if( SDL_Flip( screen ) == -1 )
            quitGame(1);

      planetImage =              loadScaledImage( "data/images/objects/planet.png", true, 255, 0, 255, scaleFactor / 10, scaleFactor / 10, 0, 0xff00ff );
      asteroidImage_1 =          loadScaledImage( "data/images/objects/asteroid.png", true, 255, 0, 255, scaleFactor / 68.846, scaleFactor / 68.846, 0, 0xff00ff );
      asteroidImage_2 =          loadScaledImage( "data/images/objects/asteroid.png", true, 255, 0, 255, scaleFactor / 43.029, scaleFactor / 43.029, 0, 0xff00ff );
      asteroidImage_3 =          loadScaledImage( "data/images/objects/asteroid.png", true, 255, 0, 255, scaleFactor / 24.588, scaleFactor / 24.588, 0, 0xff00ff );
      asteroidImage_4 =          loadScaledImage( "data/images/objects/asteroid.png", true, 255, 0, 255, scaleFactor / 17.212, scaleFactor / 17.212, 0, 0xff00ff );
      asteroidImage_5 =          loadScaledImage( "data/images/objects/asteroid.png", true, 255, 0, 255, scaleFactor / 8.196, scaleFactor / 8.196, 0, 0xff00ff );
      shipBulletImage =          loadScaledImage( "data/images/bullets/shipBullet.png", true, 255, 0, 255, scaleFactor / 60, scaleFactor / 60, 0, 0xff00ff );
      shipBulletPreviewImage =   loadScaledImage( "data/images/bullets/shipBullet.png", true, 255, 0, 255, scaleFactor / 18, scaleFactor / 18, 0, 0xff00ff );
      turretBulletImage =        loadScaledImage( "data/images/bullets/turretBullet.png", true, 255, 0, 255, scaleFactor / 60, scaleFactor / 60, 0, 0xff00ff );
      turretImage =              loadScaledImage( "data/images/objects/turret.png", true, 255, 0, 255, scaleFactor / 10, scaleFactor / 10, 0, 0xff00ff );
      crosshairsImage =          loadScaledImage( "data/images/misc/crosshairs.png", true, 255, 0, 255, scaleFactor / 10, scaleFactor / 10, 0, 0xff00ff );

      shipImages.clear();

      //Load the various images for the different colored ships available in the data/images/ships subfolder - each ship is in the format ship_001.png, ship_038.png, etc.
      while ( 1 )
      {

            testShipImage.image = NULL;
            testShipImage.previewImage = NULL;
            testShipImage.particleImage = NULL;

            //Synthesize the C string to use to load the ship image
            myStringstream.clear();
            myStringstream.str("");
            myStringstream << "data/images/ships/ship_";
            if ( index < 100 )
                  myStringstream << "0";
            if ( index < 10 )
                  myStringstream << "0";
            myStringstream << index << ".png";

            //Try to load the ship image next in line - will stop (break) when no more ships are found to load
            bigImage = load_image( myStringstream.str().c_str(), true, 255, 0, 255 );
            testImage = zoomSurface( bigImage, scaleFactor / 10, scaleFactor / 10, 0 );
            SDL_FreeSurface( bigImage );

            if ( testImage == NULL )
                  break;

            //Load the ship preview image for use while choosing ship color
            bigImage = load_image( myStringstream.str().c_str(), true, 255, 0, 255 );
            testPreviewImage = zoomSurface( bigImage, scaleFactor / 2, scaleFactor / 2, 0 );
            SDL_FreeSurface( bigImage );

            //Set color keys for the ship images
            SDL_SetColorKey( testImage, SDL_SRCCOLORKEY, 0xff00ff);
            SDL_SetColorKey( testPreviewImage, SDL_SRCCOLORKEY, 0xff00ff);

            //Set the test Ship Image object's ship images to those loaded
            testShipImage.image = testImage;
            testShipImage.previewImage = testPreviewImage;

            //Synthesize the C string to use to load the particle image
            myStringstream.clear();
            myStringstream.str("");
            myStringstream << "data/images/particles/shipExplosionParticle_";
            if ( index < 100 )
                  myStringstream << "0";
            if ( index < 10 )
                  myStringstream << "0";
            myStringstream << index << ".png";

            //Load the right colored particle
            testParticleImage = load_image( myStringstream.str().c_str(), true, 255, 255, 255 );

            //Put the particle in the Ship Image object
            testShipImage.particleImage = testParticleImage;

            //Put the newly created Ship Image into the list of Ship Images
            shipImages.push_back( testShipImage );

            //Increment the counter used to determine which ship to load
            index++;

      }

      //Check list of Ship Images to make sure everything loaded right, exit if they didn't
      for ( i = 0 ; i < shipImages.size() ; i++ )
            if ( shipImages[i].image == NULL ||
                 shipImages[i].previewImage == NULL ||
                 shipImages[i].particleImage == NULL )
                  exit ( 120 );

}

//Load all the sounds
void load_sounds()
{
      //Free the sound effects if in use
      Mix_FreeChunk( shipFire );
      Mix_FreeChunk( turretFire );
      Mix_FreeChunk( engine );
      Mix_FreeChunk( asteroidHit );

      //Load sound effects
      shipFire = Mix_LoadWAV( "data/snd/shipFire.wav" );
      turretFire = Mix_LoadWAV( "data/snd/turretFire.wav" );
      engine = Mix_LoadWAV( "data/snd/engine.wav" );
      asteroidHit = Mix_LoadWAV( "data/snd/asteroidHit.wav" );

      //If there was a problem in loading any of the sounds
      if( shipFire == NULL ||
          turretFire == NULL ||
          engine == NULL ||
          asteroidHit == NULL )
            exit ( 200 );
}

//Load all the fonts
void load_fonts()
{
      //Update the screen (actually display the new frame)
      if( SDL_Flip( screen ) == -1 )
            quitGame(1);

      //Free the fonts if in use
      TTF_CloseFont( miniFont );
      TTF_CloseFont( smallFont );
      TTF_CloseFont( largeFont );

      //Load fonts with sizes proportional to screen height
      miniFont  = TTF_OpenFont( "data/fonts/LiberationSans-Bold.ttf", ( 20 * SCREEN_HEIGHT ) / 768 );
      smallFont = TTF_OpenFont( "data/fonts/LiberationSans-Bold.ttf", ( 33 * SCREEN_HEIGHT ) / 768 );
      largeFont = TTF_OpenFont( "data/fonts/LiberationSans-Bold.ttf", ( 50 * SCREEN_HEIGHT ) / 768 );

      //If any fonts didn't load properly
      if ( miniFont == NULL ||
           smallFont == NULL ||
           largeFont == NULL )
            quitGame( 300 );
}

void load_files()
{
      //Load all fonts
      load_fonts();

      //Load all images, properly scaled if necessary
      load_images();

      //Load all sounds
      load_sounds();

}
