void applyMyParticles()
{
      int i;

      //Used for clipping sprite sheets (for particles)
      SDL_Rect clip;
      clip.y = 0;

      //Display particles from exhaust, explosions, etc.
      for ( i = 0 ; i < particleList.size() ; i++ )
      {
            clip.w = particleList[i].image->w / NUM_OF_IMAGE_FRAMES;
            clip.h = particleList[i].image->h;
            clip.x = ( particleList[i].frameInd + allFrames ) % NUM_OF_IMAGE_FRAMES * particleList[i].image->w / NUM_OF_IMAGE_FRAMES;

            applyScaledCenteredSurface( particleList[i].x,
                           MAX_SCREEN_HEIGHT - particleList[i].y,
                           particleList[i].image,
                           screen,
                           &clip );
      }
}

void applyMyObjects()
{
      int i;

      //Display objects, like asteroids or planets
      for ( i = 0 ; i < objectList.size() ; i++ )
      {
            applyScaledCenteredSurface( objectList[i].x,
                           MAX_SCREEN_HEIGHT - objectList[i].y,
                           objectList[i].image,
                           screen );
      }
}

void applyMyBullets()
{
      int i;

      //Display bullets
      for ( i = 0 ; i < bulletList.size()  ; i++ )
      {
            applyScaledCenteredSurface( bulletList[i].x,
                           MAX_SCREEN_HEIGHT - bulletList[i].y,
                           bulletList[i].image,
                           screen );
      }
}

void applyMyTurrets()
{
      int i;

      //Display turrets
      for ( i = 0 ; i < turretList.size()  ; i++ )
      {
            applyScaledCenteredSurface( turretList[i].x,
                           MAX_SCREEN_HEIGHT - turretList[i].y,
                           turretList[i].image,
                           screen );
            applyScaledCircularSurface( turretList[i].x, turretList[i].y, turretList[i].angle, turretList[i].image,
                           turretList[i].secondImage,
                           screen );
      }
}

void applyMyShips()
{
      int i;

      //Display player ships
      for ( i = 0 ; i < playerList.size() ; i++ )
      {
            //Display ship if it is not dead, OR skip/draw a frame for the ship depending on status of invincibility
            if ( playerList[i].theShip.exist && playerList[i].lives > 0 )
            {
                  //If ship is invincible, make image flash
                  if ( ( !playerList[i].theShip.invincible || allFrames % 2 ) || gamePaused )
                  {
                        applyScaledCenteredSurface( playerList[i].theShip.x,
                                       MAX_SCREEN_HEIGHT - playerList[i].theShip.y,
                                       shipImages[ playerList[i].theShip.colorNum ].image,
                                       screen );
                        applyScaledCircularSurface( playerList[i].theShip.x, playerList[i].theShip.y, playerList[i].theShip.angle,
                                       shipImages[ playerList[i].theShip.colorNum ].image,
                                       playerList[i].theShip.secondImage,
                                       screen );
                  }
            }
      }
}

void applyMyText()
{
      int i,
          eventNoticeListSize = eventNoticeList.size();

      for ( i = 0 ; i < eventNoticeListSize ; i++ )
      {
            if ( i > 1 )
                  applySurface( SCREEN_WIDTH / 160.0, ( eventNoticeList[ i - 1 ].notice->h + SCREEN_HEIGHT / 160.0 ) * i - ( eventNoticeList[ i - 1 ].notice->h ) + eventNoticeList[i].notice->h, eventNoticeList[i].notice, screen );
            else
                  applySurface( SCREEN_WIDTH / 160.0, ( eventNoticeList[ i ].notice->h + SCREEN_HEIGHT / 160.0 ) * i - ( eventNoticeList[ i ].notice->h ) + eventNoticeList[i].notice->h, eventNoticeList[i].notice, screen );
      }
}

void applyMyCrosshairs()
{
      int i;

      for ( i = 0 ; i < turretList.size()  ; i++ )
            if ( playerList[ turretList[i].playerWithClosestShip ].theShip.exist )
                  applyScaledCenteredSurface( turretList[i].x + turretList[i].projX,
                                              MAX_SCREEN_HEIGHT - ( turretList[i].y + turretList[i].projY ),
                                              crosshairsImage,
                                              screen );
}

void applySurfaces( int applyParticles, int applyObjects, int applyBullets, int applyTurrets, int applyShips, int applyText, int applyCrosshairs, int applyFPS )
{
      //Display the background
      applySurface( 0, 0, background, screen );

      if ( displayFPS && applyFPS )
      {
            myStringstream.clear();
            myStringstream.str("");
            myStringstream << "FPS: " << displayedRealFPS;

            text = TTF_RenderText_Solid( miniFont, myStringstream.str().c_str(), blackColour );
            applySurface( SCREEN_WIDTH - 5 - text->w, 5, text, screen );
            SDL_FreeSurface( text );
      }

      if ( applyText )
            applyMyText();

      if ( applyParticles )
            applyMyParticles();

      if ( applyObjects )
            applyMyObjects();

      if ( applyBullets )
            applyMyBullets();

      if ( applyTurrets )
            applyMyTurrets();

      //If parameter applyShips is true, then display the ships
      if ( applyShips )
            applyMyShips();

      //Display crosshairs for turrets if asked to and if game is set to display them
      if ( applyCrosshairs && displayCrosshairs )
            applyMyCrosshairs();
}
