int choosePlayers()
{
      int choosingPlayer = true,
          fpsTimer;

      while ( choosingPlayer )
      {
            //Start the frame timer
            fpsTimer = SDL_GetTicks();

            //Display the background
            applySurface( 0, 0, background, screen );

            //Update asteroids
            objectListUpdate();
            eventNoticeUpdate();
            checkBounds( false );

            //Reapply surfaces
            applySurfaces( 0, 1, 0, 1, 0, 1, 0, 0 );

            //Display the title
            displayTitle();

            myStringstream.clear();
            myStringstream.str("");
            myStringstream << "How Many Players? " << theWorld.numOfPlayers;

            text = TTF_RenderText_Solid( largeFont, myStringstream.str().c_str(), blackColour );
            applySurface( SCREEN_WIDTH / 2 - text->w/2 + 4, SCREEN_HEIGHT / 2 - 2 * text->h, text, screen );
            SDL_FreeSurface( text );

            //Toggle fullscreen if user pressed alt-enter
            keystates = SDL_GetKeyState( NULL );
            checkFullscreenToggle();

            //While there's events to handle
            while( SDL_PollEvent( &event ) )
            {
                  //If the user has Xed out the window
                  if ( event.type == SDL_QUIT )
                  {
                        //Quit the program
                        quitGame( 0 );
                  }

                  if ( event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_UP )
                  {
                        theWorld.numOfPlayers++;
                  }

                  if ( event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_DOWN && theWorld.numOfPlayers > 1 )
                  {
                        theWorld.numOfPlayers--;
                  }

                  if ( event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_RETURN && !keystates[ SDLK_LALT ] && !keystates[ SDLK_RALT ] )
                        choosingPlayer = false;

                  //If user pressed escape, quit to main menu
                  if ( event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_ESCAPE )
                        return -1;
            }

            //Update the screen (actually display the new frame)
            if( SDL_Flip( screen ) == -1 )
                  return -1;

            //Cap the frame rate
            if( ( SDL_GetTicks() - fpsTimer ) < 1000 / (float)FRAMES_PER_SECOND && capFramerate )
            {
                  SDL_Delay( ( 1000 / (float)FRAMES_PER_SECOND ) - SDL_GetTicks() + fpsTimer );
            }
      }

      return 0;
}

int chooseControlMethod()
{
      int fpsTimer,
          choosingControlMethod,
          choosingShipControls,
          spotOnMenu,
          numOfJoysticks,
          numOfButtons,
          keyWasPressed,
          keymapArray[5],
          i, j, k;

      for ( i = 0 ; i < theWorld.numOfPlayers ; i++ )
      {
            spotOnMenu = 0;
            choosingControlMethod = true;
            choosingShipControls = true;
            numOfJoysticks = SDL_NumJoysticks();

            while ( choosingControlMethod && numOfJoysticks > 0 )
            {
                  //Continue to refresh number of joystick devices
                  numOfJoysticks = SDL_NumJoysticks();

                  //Start the frame timer
                  fpsTimer = SDL_GetTicks();

                  //Display the background
                  applySurface( 0, 0, background, screen );

                  //Update asteroids
                  objectListUpdate();
                  eventNoticeUpdate();
                  checkBounds( false );

                  //Reapply surfaces
                  applySurfaces( 0, 1, 0, 1, 0, 1, 0, 0 );

                  //Display the title
                  displayTitle();

                  myStringstream.clear();
                  myStringstream.str("");
                  myStringstream << "What device will player " << i + 1 << " use?" ;

                  text = TTF_RenderText_Solid( largeFont, myStringstream.str().c_str(), blackColour );
                  applySurface( SCREEN_WIDTH / 2 - text->w/2, SCREEN_HEIGHT / 2 - 2 * text->h, text, screen );
                  SDL_FreeSurface( text );

                  if ( spotOnMenu == 0 )
                  {
                        text = TTF_RenderText_Solid( smallFont, "Keyboard", blackColour );
                        applySurface( SCREEN_WIDTH / 2 - text->w/2, SCREEN_HEIGHT / 2 - text->h / 2.0, text, screen );
                        SDL_FreeSurface( text );
                  }
                  else
                  {
                        myStringstream.clear();
                        myStringstream.str("");
                        myStringstream << SDL_JoystickName( spotOnMenu - 1 ) << " (joystick " << spotOnMenu << ")";

                        text = TTF_RenderText_Solid( smallFont, myStringstream.str().c_str(), blackColour );
                        applySurface( SCREEN_WIDTH / 2 - text->w/2, SCREEN_HEIGHT / 2 - text->h / 2.0, text, screen );
                        SDL_FreeSurface( text );
                  }

                  //Toggle fullscreen if user pressed alt-enter
                  keystates = SDL_GetKeyState( NULL );
                  checkFullscreenToggle();

                  //While there's events to handle
                  while( SDL_PollEvent( &event ) )
                  {
                        //If the user has Xed out the window
                        if ( event.type == SDL_QUIT )
                        {
                              //Quit the program
                              quitGame( 0 );
                        }

                        if ( event.type == SDL_KEYDOWN && ( event.key.keysym.sym == SDLK_DOWN || event.key.keysym.sym == SDLK_LEFT ) )
                        {
                              spotOnMenu++;
                        }

                        if ( event.type == SDL_KEYDOWN && ( event.key.keysym.sym == SDLK_UP || event.key.keysym.sym == SDLK_RIGHT ) )
                        {
                              spotOnMenu--;
                        }

                        if ( event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_RETURN && !keystates[ SDLK_LALT ] && !keystates[ SDLK_RALT ] )
                              choosingControlMethod = false;

                        //If user pressed escape, quit to main menu
                        if ( event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_ESCAPE )
                              return -1;
                  }

                  if ( spotOnMenu > numOfJoysticks )
                        spotOnMenu = 0;
                  if ( spotOnMenu < 0 )
                        spotOnMenu = numOfJoysticks;

                  //Update the screen (actually display the new frame)
                  if( SDL_Flip( screen ) == -1 )
                        return -1;

                  //Cap the frame rate
                  if( ( SDL_GetTicks() - fpsTimer ) < 1000 / (float)FRAMES_PER_SECOND && capFramerate )
                  {
                        SDL_Delay( ( 1000 / (float)FRAMES_PER_SECOND ) - SDL_GetTicks() + fpsTimer );
                  }
            }

            //Determine which control device the player chose (if choice was allowed) and assign as necessary
            if ( spotOnMenu == 0 )
                  playerList[i].joystickControlsShip = false;
            else
            {
                  playerList[i].assignJoystick( spotOnMenu - 1 );
                  playerList[i].joystickControlsShip = true;
            }

            //If keyboard was chosen to use for input, assign controls for keyboard
            if ( !playerList[i].joystickControlsShip )
            {
                  for ( j = 0 ; j < 5 ; j++ )
                  {
                        keyWasPressed = false;

                        while ( !keyWasPressed )
                        {
                              //Start the frame timer
                              fpsTimer = SDL_GetTicks();

                              //Display the background
                              applySurface( 0, 0, background, screen );

                              //Update asteroids
                              objectListUpdate();
                              eventNoticeUpdate();
                              checkBounds( false );

                              //Reapply surfaces
                              applySurfaces( 0, 1, 0, 1, 0, 1, 0, 0 );

                              //Display the title
                              displayTitle();

                              myStringstream.clear();
                              myStringstream.str("");
                              myStringstream << "Press key player " << i + 1 << " will use for ";

                              switch (j)
                              {
                                    case 0:
                                          myStringstream << "Left";
                                          break;
                                    case 1:
                                          myStringstream << "Right";
                                          break;
                                    case 2:
                                          myStringstream << "Up";
                                          break;
                                    case 3:
                                          myStringstream << "Down";
                                          break;
                                    case 4:
                                          myStringstream << "Fire";
                                          break;
                                    default:
                                          myStringstream << "[Error]";
                                          break;
                              }

                              text = TTF_RenderText_Solid( largeFont, myStringstream.str().c_str(), blackColour );
                              applySurface( SCREEN_WIDTH / 2 - text->w/2, SCREEN_HEIGHT / 2 - 2 * text->h, text, screen );
                              SDL_FreeSurface( text );

                              //While there's events to handle
                              while( SDL_PollEvent( &event ) )
                              {
                                    //If the user has Xed out the window
                                    if ( event.type == SDL_QUIT )
                                    {
                                          //Quit the program
                                          quitGame( 0 );
                                    }

                                    //If user pressed escape, quit to main menu
                                    if ( event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_ESCAPE )
                                          return -1;

                                    if ( event.type == SDL_KEYDOWN && event.key.keysym.sym != SDLK_PAUSE )
                                    {
                                          keymapArray[j] = event.key.keysym.sym;
                                          keyWasPressed = true;
                                    }
                              }

                              //Update the screen (actually display the new frame)
                              if ( SDL_Flip( screen ) == -1 )
                                    return -1;

                              //Cap the frame rate
                              if ( ( SDL_GetTicks() - fpsTimer ) < 1000 / (float)FRAMES_PER_SECOND && capFramerate )
                              {
                                    SDL_Delay( ( 1000 / (float)FRAMES_PER_SECOND ) - SDL_GetTicks() + fpsTimer );
                              }
                        }
                  }

                  //Assign keys that were pressed to proper spots in keymap class
                  playerList[i].myKeymap.kbLeft = keymapArray[0];
                  playerList[i].myKeymap.kbRight = keymapArray[1];
                  playerList[i].myKeymap.kbUp = keymapArray[2];
                  playerList[i].myKeymap.kbDown = keymapArray[3];
                  playerList[i].myKeymap.kbFire = keymapArray[4];
            }
            else
            {
                  for ( j = 0 ; j < 5 ; j++ )
                  {
                        //Assign no button to joystick fire button
                        playerList[i].joystickFireButton = -1;

                        while ( playerList[i].joystickFireButton == -1 )
                        {
                              //Start the frame timer
                              fpsTimer = SDL_GetTicks();

                              //Display the background
                              applySurface( 0, 0, background, screen );

                              //Update asteroids
                              objectListUpdate();
                              eventNoticeUpdate();
                              checkBounds( false );

                              //Reapply surfaces
                              applySurfaces( 0, 1, 0, 1, 0, 1, 0, 0 );

                              //Display the title
                              displayTitle();

                              myStringstream.clear();
                              myStringstream.str("");
                              myStringstream << "Press button player " << i + 1 << " will use for Fire";

                              text = TTF_RenderText_Solid( largeFont, myStringstream.str().c_str(), blackColour );
                              applySurface( SCREEN_WIDTH / 2 - text->w/2, SCREEN_HEIGHT / 2 - 2 * text->h, text, screen );
                              SDL_FreeSurface( text );

                              //While there's events to handle
                              while( SDL_PollEvent( &event ) )
                              {
                                    //If the user has Xed out the window
                                    if ( event.type == SDL_QUIT )
                                    {
                                          //Quit the program
                                          quitGame( 0 );
                                    }

                                    //If user pressed escape, quit to main menu
                                    if ( event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_ESCAPE )
                                          return -1;
                              }

                              numOfButtons = SDL_JoystickNumButtons( playerList[i].joystick );

                              for ( k = 0 ; k < numOfButtons && playerList[i].joystickFireButton == -1 ; k++ )
                              {
                                    if ( SDL_JoystickGetButton( playerList[i].joystick, k ) ) playerList[i].joystickFireButton = k;
                              }

                              //Update the screen (actually display the new frame)
                              if ( SDL_Flip( screen ) == -1 )
                                    return -1;

                              //Cap the frame rate
                              if ( ( SDL_GetTicks() - fpsTimer ) < 1000 / (float)FRAMES_PER_SECOND && capFramerate )
                              {
                                    SDL_Delay( ( 1000 / (float)FRAMES_PER_SECOND ) - SDL_GetTicks() + fpsTimer );
                              }
                        }
                  }
            }
      }

      return 0;
}

int chooseDifficulty()
{
      int choosingDifficulty = true,
          fpsTimer;

      //Reset level counter and difficulty
      theWorld.currentLevelNumber = 1;
      theWorld.overallDifficulty = 1;

      while ( choosingDifficulty )
      {
            //Start the frame timer
            fpsTimer = SDL_GetTicks();

            //Display the background
            applySurface( 0, 0, background, screen );

            //Update asteroids
            objectListUpdate();
            eventNoticeUpdate();
            checkBounds( false );

            //Reapply surfaces
            applySurfaces( 0, 1, 0, 1, 0, 1, 0, 0 );

            //Display the title

            displayTitle();

            text = TTF_RenderText_Solid( smallFont, "What level of difficulty do you want?", blackColour );
            applySurface( SCREEN_WIDTH / 2 - text->w / 2, SCREEN_HEIGHT / 2 - 3 * text->h / 2, text, screen );
            SDL_FreeSurface( text );
            text = TTF_RenderText_Solid( smallFont, "1 is the easiest, 4+ is ludicrous", blackColour );
            applySurface( SCREEN_WIDTH / 2 - text->w / 2, SCREEN_HEIGHT / 2 - text->h / 2, text, screen );
            SDL_FreeSurface( text );

            myStringstream.clear();
            myStringstream.str("");
            myStringstream << theWorld.overallDifficulty;

            text = TTF_RenderText_Solid( largeFont, myStringstream.str().c_str(), blackColour );
            applySurface( SCREEN_WIDTH / 2 - text->w/2 + 4, SCREEN_HEIGHT / 2 + 1 * text->h / 2, text, screen );
            SDL_FreeSurface( text );

            //Toggle fullscreen if user pressed alt-enter
            keystates = SDL_GetKeyState( NULL );
            checkFullscreenToggle();

            //While there's events to handle
            while( SDL_PollEvent( &event ) )
            {
                  //If the user has Xed out the window
                  if ( event.type == SDL_QUIT )
                  {
                        //Quit the program
                        quitGame( 0 );
                  }

                  if ( event.type == SDL_KEYDOWN && ( event.key.keysym.sym == SDLK_UP || event.key.keysym.sym == SDLK_RIGHT ) )
                  {
                        theWorld.overallDifficulty++;
                  }

                  if ( event.type == SDL_KEYDOWN && ( event.key.keysym.sym == SDLK_DOWN || event.key.keysym.sym == SDLK_LEFT ) && theWorld.overallDifficulty > 1 )
                  {
                        theWorld.overallDifficulty--;
                  }

                  if ( event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_RETURN && !keystates[ SDLK_LALT ] && !keystates[ SDLK_RALT ] )
                        choosingDifficulty = false;

                  //If user pressed escape, quit to main menu
                  if ( event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_ESCAPE )
                        return -1;
            }

            //Update the screen (actually display the new frame)
            if( SDL_Flip( screen ) == -1 )
                  quitGame(1);

            //Cap the frame rate
            if( ( SDL_GetTicks() - fpsTimer ) < 1000 / (float)FRAMES_PER_SECOND && capFramerate )
            {
                  SDL_Delay( ( 1000 / (float)FRAMES_PER_SECOND ) - SDL_GetTicks() + fpsTimer );
            }

      }

      return true;
}

int chooseShipColor()
{
      int fpsTimer,
          choosingShipColor,
          shipChosen,
          i;

      double shipAngle = 0;

      for ( i = 0 ; i < theWorld.numOfPlayers ; i++ )
      {
            shipChosen = 0;
            choosingShipColor = true;

            while ( choosingShipColor )
            {
                  //Start the frame timer
                  fpsTimer = SDL_GetTicks();

                  //Display the background
                  applySurface( 0, 0, background, screen );

                  //Update asteroids
                  objectListUpdate();
                  eventNoticeUpdate();
                  checkBounds( false );

                  //Reapply surfaces
                  applySurfaces( 0, 1, 0, 1, 0, 1, 0, 0 );

                  //Display the title
                  displayTitle();

                  myStringstream.clear();
                  myStringstream.str("");
                  myStringstream << "What ship will player " << i + 1 << " use?" ;

                  text = TTF_RenderText_Solid( largeFont, myStringstream.str().c_str(), blackColour );
                  applySurface( SCREEN_WIDTH / 2 - text->w/2, SCREEN_HEIGHT / 2 - 3.5 * text->h, text, screen );
                  SDL_FreeSurface( text );

                  applySurface( SCREEN_WIDTH / 2 - shipImages[shipChosen].previewImage->w / 2,
                                 SCREEN_HEIGHT / 2 - shipImages[shipChosen].previewImage->h / 2,
                                 shipImages[shipChosen].previewImage,
                                 screen );
                  applySurface( ( SCREEN_WIDTH / 2 ) - shipBulletPreviewImage->w / 2 + ( shipImages[shipChosen].previewImage->w / 2 ) * cos( shipAngle ),
                                 ( SCREEN_HEIGHT / 2 ) - shipBulletPreviewImage->h / 2 + ( shipImages[shipChosen].previewImage->h / 2 ) * sin( shipAngle ),
                                 shipBulletPreviewImage,
                                 screen );

                  shipAngle+= .05;
                  if ( shipAngle > 360 )
                        shipAngle -= 360;

                  //Toggle fullscreen if user pressed alt-enter
                  keystates = SDL_GetKeyState( NULL );
                  checkFullscreenToggle();

                  //While there's events to handle
                  while( SDL_PollEvent( &event ) )
                  {
                        //If the user has Xed out the window
                        if ( event.type == SDL_QUIT )
                        {
                              //Quit the program
                              quitGame( 0 );
                        }

                        if ( event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_LEFT )
                        {
                              shipChosen--;
                              if ( shipChosen < 0 )
                                    shipChosen = shipImages.size() - 1;
                        }

                        if ( event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_RIGHT )
                        {
                              shipChosen++;
                              if ( shipChosen > shipImages.size() - 1 )
                                    shipChosen = 0;
                        }

                        if ( event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_RETURN && !keystates[ SDLK_LALT ] && !keystates[ SDLK_RALT ] )
                              choosingShipColor = false;

                        //If user pressed escape, quit to main menu
                        if ( event.type == SDL_KEYDOWN && event.key.keysym.sym == SDLK_ESCAPE )
                              return -1;
                  }

                  //Update the screen (actually display the new frame)
                  if( SDL_Flip( screen ) == -1 )
                        quitGame( 1 );

                  //Cap the frame rate
                  if( ( SDL_GetTicks() - fpsTimer ) < 1000 / (float)FRAMES_PER_SECOND && capFramerate )
                  {
                        SDL_Delay( ( 1000 / (float)FRAMES_PER_SECOND ) - SDL_GetTicks() + fpsTimer );
                  }
            }

            playerList[i].theShip.colorNum = shipChosen;
      }
}

//Returns -1 if user hit escape at any point, returns 0 otherwise
int configureGame()
{
      //Choose how many players to have in the game
      if ( choosePlayers() == -1 )
            return -1;

      //Create the list of players with default settings (they will be configured in next steps)
      createPlayerList();

      //Choose whether to control player with keyboard or joystick (if joystick present) and assign buttons
      if ( chooseControlMethod() == -1 )
            return -1;

      if ( chooseShipColor() == -1 )
            return -1;

      return 0;
}
