void objectListUpdate()
{
      int i, j,                                 //Looping variables
          objectListSize = objectList.size(),   //Size of object list
          bulletListSize = bulletList.size();   //Size of bullet list

      float a, b, d, e, g, h, m, acceleration, xVelPre, yVelPre;

      //For each object in the object list
      for ( i = 0 ; i < objectListSize ; i++ )
      {
            //If object exists
            if ( objectList[i].exist )
            {
                  xVelPre = 0;
                  yVelPre = 0;

                  //If object is not fixed in place (it can move) and is affected by gravity
                  if ( objectList[i].canMove && objectList[i].canBePulled )
                  {
                        //Again, for each object (j) in the list
                        for ( j = 0 ; j < objectListSize ; j++ )
                        {
                              a = objectList[i].x;
                              b = objectList[i].y;
                              d = objectList[i].xVel;
                              e = objectList[i].yVel;

                              //If object i is not looking at itself and object j has gravitational pull
                              if ( i != j && objectList[j].hasPull )
                              {
                                    g = objectList[j].x;
                                    h = objectList[j].y;
                                    m = ( ( (g-a) * (g-a) ) + ( (h-b) * (h-b) ) );

                                    if ( m != 0 )
                                    {
                                          xVelPre += objectList[i].mass * objectList[j].mass * (g-a) / ( 14 * m + 5 );
                                          yVelPre += objectList[i].mass * objectList[j].mass * (h-b) / ( 14 * m + 5 );
                                    }
                              }
                        }

                        //For each bullet in list
                        for ( j = 0 ; j < bulletListSize ; j++ )
                        {
                              //If bullet has pull
                              if ( bulletList[j].hasPull )
                              {
                                    g = bulletList[j].x;
                                    h = bulletList[j].y;
                                    m = ( ( (g-a) * (g-a) ) + ( (h-b) * (h-b) ) );

                                    if ( m != 0 )
                                    {
                                          xVelPre += objectList[i].mass * bulletList[j].mass * (g-a) / ( 14 * m + 5 );
                                          yVelPre += objectList[i].mass * bulletList[j].mass * (h-b) / ( 14 * m + 5 );
                                    }
                              }
                        }

                        acceleration = sqrt( ( xVelPre * xVelPre ) + ( yVelPre * yVelPre ) );

                        //Cap acceleration
                        if ( acceleration > MAX_ACCELERATION * speedMod * ratioMult )
                        {
                              xVelPre = MAX_ACCELERATION * ( xVelPre / acceleration );
                              yVelPre = MAX_ACCELERATION * ( yVelPre / acceleration );
                        }

                  }

                  //Add object acceleration vector to velocity vector
                  objectList[i].xVel += xVelPre * ratioMult;
                  objectList[i].yVel += yVelPre * ratioMult;

                  //Add object velocity vector to position vector
                  objectList[i].x += objectList[i].xVel * speedMod * ratioMult;
                  objectList[i].y += objectList[i].yVel * speedMod * ratioMult;

             }
      }
}

void bulletListUpdate()
{
      int i, j,
          objectListSize = objectList.size(),
          bulletListSize = bulletList.size();

      float a, b, d, e, g, h, m, acceleration, xVelPre, yVelPre;

      //For each bullet in the bullet list
      for ( i = 0 ; i < bulletListSize ; i++ )
      {
            //If bullet is dead
            if ( ( SDL_GetTicks() - bulletList[i].startTime ) * speedMod > bulletList[i].maxLife )
            {
                  bulletList[i].exist = 0;
            }
            else
            {
                  xVelPre = 0;
                  yVelPre = 0;
/*
                  //Shorten life of bullet according to speed of game
                  bulletList[i].realTicksLeft -= speedMod *
*/
                  //If object is affected by gravity, or if bullet can be pulled by asteroids
                  if ( bulletList[i].canBePulled || bulletList[i].pulledByAsteroids )
                  {
                        a = bulletList[i].x;
                        b = bulletList[i].y;
                        d = bulletList[i].xVel;
                        e = bulletList[i].yVel;

                        //Again, for each object (j) in the list
                        for ( j = 0 ; j < objectListSize ; j++ )
                        {
                              //If object has pull or object is asteroid and bullet is attracted to asteroids
                              if ( objectList[j].hasPull || ( bulletList[i].pulledByAsteroids && objectList[j].name == ASTEROID ) )
                              {
                                    g = objectList[j].x;
                                    h = objectList[j].y;
                                    m = ( ( (g-a) * (g-a) ) + ( (h-b) * (h-b) ) );

                                    if ( m != 0 )
                                    {
                                          //If Bullet is being pulled by an asteroid, increase pull as if bullets were tracking asteroids
                                          if ( bulletList[i].pulledByAsteroids && objectList[j].name == ASTEROID )
                                          {
                                                xVelPre += ( 500.0 ) * (g-a) / ( 14 * m + 5 );
                                                yVelPre += ( 500.0 ) * (h-b) / ( 14 * m + 5 );
                                          }
                                          //Otherwise, pull as would be normally
                                          else if ( bulletList[i].canBePulled )
                                          {
                                                xVelPre += objectList[j].mass * (g-a) / ( 14 * m + 5 );
                                                yVelPre += objectList[j].mass * (h-b) / ( 14 * m + 5 );
                                          }
                                    }
                              }
                        }

                        //For each bullet in list
                        for ( j = 0 ; j < bulletListSize ; j++ )
                        {
                              //If object i is not looking at itself and object has pull
                              if ( i != j && bulletList[j].hasPull )
                              {
                                    g = bulletList[j].x;
                                    h = bulletList[j].y;

                                    m = ( ( (g-a) * (g-a) ) + ( (h-b) * (h-b) ) );

                                    xVelPre += bulletList[j].mass * (g-a) / ( 14 * m + 5 );
                                    yVelPre += bulletList[j].mass * (h-b) / ( 14 * m + 5 );
                              }
                        }
                  }

                  acceleration = sqrt( ( xVelPre * xVelPre ) + ( yVelPre * yVelPre ) );

                  //Cap acceleration
                  if ( acceleration > MAX_ACCELERATION * speedMod * ratioMult )
                  {
                        xVelPre = MAX_ACCELERATION * ( xVelPre / acceleration );
                        yVelPre = MAX_ACCELERATION * ( yVelPre / acceleration );
                  }

                  //Add bullet acceleration vector to velocity vector
                  bulletList[i].xVel += xVelPre * ratioMult;
                  bulletList[i].yVel += yVelPre * ratioMult;

                  //Add bullet velocity vector to position vector
                  bulletList[i].x += bulletList[i].xVel * speedMod * ratioMult;
                  bulletList[i].y += bulletList[i].yVel * speedMod * ratioMult;
            }
      }
}

void shipUpdate()
{
      int i, j,
          objectListSize = objectList.size(),
          bulletListSize = bulletList.size(),
          playerListSize = playerList.size(),
          joyXAxis, joyYAxis, hatMovedShip,
          stateOfHats,
          fireABullet;

      float a, b, d, e, g, h, m,
            acceleration,
            xVelPre, yVelPre;

      //for all the players in the player list
      for ( j = 0 ; j < playerListSize ; j++ )
      {
            //If ship is alive and has lives left
            if ( playerList[j].theShip.exist && playerList[j].lives > 0 )
            {
                  fireABullet = false;

                  //Test if ship just regenerated or not, and if so whether it is vulnerable or not
                  if ( playerList[j].theShip.justRegenerated && ( SDL_GetTicks() - playerList[j].theShip.timeOfRegen ) * speedMod > playerList[j].theShip.lengthOfNewness )
                  {
                        playerList[j].theShip.invincible = false;
                        playerList[j].theShip.justRegenerated = false;
                  }

                  a = playerList[j].theShip.x;
                  b = playerList[j].theShip.y;
                  d = playerList[j].theShip.xVel;
                  e = playerList[j].theShip.yVel;

                  xVelPre = 0;
                  yVelPre = 0;

                  //For each object (j) in the list
                  for ( i = 0 ; i < objectListSize ; i++ )
                  {
                        //If object j has gravitational pull
                        if ( objectList[i].hasPull )
                        {
                              g = objectList[i].x;
                              h = objectList[i].y;
                              m = ( ( (g-a) * (g-a) ) + ( (h-b) * (h-b) ) );

                              if ( m != 0 )
                              {
                                    xVelPre += objectList[i].mass * (g-a) / ( 14 * m + 5 );
                                    yVelPre += objectList[i].mass * (h-b) / ( 14 * m + 5 );
                              }
                        }
                  }

                  //For each bullet in list
                  for ( i = 0 ; i < bulletListSize ; i++ )
                  {
                        //If bullet has pull
                        if ( bulletList[i].hasPull )
                        {
                              g = bulletList[i].x;
                              h = bulletList[i].y;
                              m = ( ( (g-a) * (g-a) ) + ( (h-b) * (h-b) ) );

                              if ( m != 0 )
                              {
                                    xVelPre += bulletList[i].mass * (g-a) / ( 14 * m + 5 );
                                    yVelPre += bulletList[i].mass * (h-b) / ( 14 * m + 5 );
                              }
                        }
                  }

                  keystates = SDL_GetKeyState( NULL );

                  //If playerList[j].joystick is NOT being used to control ship, check keyboard for ship control buttons being pressed
                  if ( !playerList[j].joystickControlsShip )
                  {
                        //If up is pressed
                        if( keystates[ playerList[j].myKeymap.kbUp ] )
                        {
                              if ( playSounds )
                                    Mix_PlayChannel( -1, engine, 0 );
                              xVelPre += playerList[j].theShip.thrust * cos( playerList[j].theShip.angle );
                              yVelPre += playerList[j].theShip.thrust * sin( playerList[j].theShip.angle );

                              //If necessary, create particle of exaust
                              if ( ( SDL_GetTicks() -  playerList[j].theShip.particleTime ) > playerList[j].theShip.particleRechargeTime )playerList[j].theShip.createParticleUp();
                        }

                        //If down is pressed
                        if( keystates[ playerList[j].myKeymap.kbDown ] )
                        {
                              if ( playSounds )
                                    Mix_PlayChannel( -1, engine, 0 );
                              xVelPre -= playerList[j].theShip.thrust * cos( playerList[j].theShip.angle );
                              yVelPre -= playerList[j].theShip.thrust * sin( playerList[j].theShip.angle );

                              //If necessary, create particle of exaust
                              if ( ( SDL_GetTicks() -  playerList[j].theShip.particleTime ) > playerList[j].theShip.particleRechargeTime ) playerList[j].theShip.createParticleDown();
                        }

                        //If left is pressed
                        if( keystates[ playerList[j].myKeymap.kbLeft ] )
                              playerList[j].theShip.angle += playerList[j].theShip.rotateSpeed * speedMod * ratioMult;

                        //If right is pressed
                        if( keystates[ playerList[j].myKeymap.kbRight ] )
                              playerList[j].theShip.angle -= playerList[j].theShip.rotateSpeed * speedMod * ratioMult;

                        //If fire button is pressed
                        if( keystates[ playerList[j].myKeymap.kbFire ] && ( SDL_GetTicks() - playerList[j].theShip.bulletTime ) * speedMod  * ( realFPS / FRAMES_PER_SECOND ) > playerList[j].theShip.shotRechargeTime )
                              fireABullet = true;
                  }
                  else
                  {
                        //assume that hat was not used until proved otherwise
                        hatMovedShip = false;

                        //Modify ship motion using hat if hat was used
                        if ( playerList[j].joyNumOfHats > 0 );
                        {
                              stateOfHats = SDL_JoystickGetHat( playerList[j].joystick, 0 );

                              if ( stateOfHats == SDL_HAT_UP || stateOfHats == SDL_HAT_LEFTUP || stateOfHats == SDL_HAT_RIGHTUP )
                              {
                                    if ( playSounds )
                                          Mix_PlayChannel( -1, engine, 0 );
                                    xVelPre += playerList[j].theShip.thrust * cos( playerList[j].theShip.angle );
                                    yVelPre += playerList[j].theShip.thrust * sin( playerList[j].theShip.angle );

                                    //If necessary, create particle of exaust
                                    if ( ( SDL_GetTicks() -  playerList[j].theShip.particleTime ) > playerList[j].theShip.particleRechargeTime )playerList[j].theShip.createParticleUp();

                                    hatMovedShip = true;
                              }
                              else if ( stateOfHats == SDL_HAT_DOWN || stateOfHats == SDL_HAT_LEFTDOWN || stateOfHats == SDL_HAT_RIGHTDOWN )
                              {
                                    if ( playSounds )
                                          Mix_PlayChannel( -1, engine, 0 );
                                    xVelPre -= playerList[j].theShip.thrust * cos( playerList[j].theShip.angle );
                                    yVelPre -= playerList[j].theShip.thrust * sin( playerList[j].theShip.angle );

                                    //If necessary, create particle of exaust
                                    if ( ( SDL_GetTicks() -  playerList[j].theShip.particleTime ) > playerList[j].theShip.particleRechargeTime )playerList[j].theShip.createParticleDown();

                                    hatMovedShip = true;
                              }

                              if ( stateOfHats == SDL_HAT_LEFT || stateOfHats == SDL_HAT_LEFTUP || stateOfHats == SDL_HAT_LEFTDOWN )
                              {
                                          playerList[j].theShip.angle += playerList[j].theShip.rotateSpeed * speedMod * ratioMult;
                                          hatMovedShip = true;
                              }
                              else if ( stateOfHats == SDL_HAT_RIGHT || stateOfHats == SDL_HAT_RIGHTUP || stateOfHats == SDL_HAT_RIGHTDOWN )
                              {
                                          playerList[j].theShip.angle -= playerList[j].theShip.rotateSpeed * speedMod * ratioMult;
                                          hatMovedShip = true;
                              }
                        }

                        //Modify ship motion using axis if hat was not used
                        if ( hatMovedShip == false )
                        {
                              joyXAxis = SDL_JoystickGetAxis( playerList[j].joystick, 0 );
                              joyYAxis = SDL_JoystickGetAxis( playerList[j].joystick, 1 );

                              if ( joyXAxis < -1 * playerList[j].joystickDeadzone )
                                    playerList[j].theShip.angle -= ( joyXAxis / 32768.0 ) * playerList[j].theShip.rotateSpeed * speedMod * ratioMult;
                              else if ( joyXAxis > playerList[j].joystickDeadzone )
                                    playerList[j].theShip.angle -= ( joyXAxis / 32768.0 ) * playerList[j].theShip.rotateSpeed * speedMod * ratioMult;

                              if ( joyYAxis < -1 * playerList[j].joystickDeadzone )
                              {
                                    if ( playSounds )
                                          Mix_PlayChannel( -1, engine, 0 );
                                    xVelPre -= ( joyYAxis / 32768.0 ) * playerList[j].theShip.thrust * cos( playerList[j].theShip.angle );
                                    yVelPre -= ( joyYAxis / 32768.0 ) * playerList[j].theShip.thrust * sin( playerList[j].theShip.angle );

                                    //If necessary, create particle of exaust
                                    if ( ( SDL_GetTicks() -  playerList[j].theShip.particleTime ) * speedMod > playerList[j].theShip.particleRechargeTime ) playerList[j].theShip.createParticleUp();
                              }
                              else if ( joyYAxis > playerList[j].joystickDeadzone )
                              {
                                    if ( playSounds )
                                          Mix_PlayChannel( -1, engine, 0 );
                                    xVelPre -= ( joyYAxis / 32768.0 ) * playerList[j].theShip.thrust * cos( playerList[j].theShip.angle );
                                    yVelPre -= ( joyYAxis / 32768.0 ) * playerList[j].theShip.thrust * sin( playerList[j].theShip.angle );

                                    //If necessary, create particle of exaust
                                    if ( ( SDL_GetTicks() -  playerList[j].theShip.particleTime ) > playerList[j].theShip.particleRechargeTime )playerList[j].theShip.createParticleDown();
                              }
                        }

                        //Check if fire button was pressed
                        if ( SDL_JoystickGetButton( playerList[j].joystick, playerList[j].joystickFireButton ) && ( SDL_GetTicks() - playerList[j].theShip.bulletTime ) * speedMod  * ( realFPS / FRAMES_PER_SECOND ) > playerList[j].theShip.shotRechargeTime )
                        {
                              fireABullet = true;
                        }
                  }

                  acceleration = sqrt( ( xVelPre * xVelPre ) + ( yVelPre * yVelPre ) );

                  //Cap acceleration
                  if ( acceleration > MAX_ACCELERATION * speedMod * ratioMult )
                  {
                        xVelPre = MAX_ACCELERATION * ( xVelPre / acceleration );
                        yVelPre = MAX_ACCELERATION * ( yVelPre / acceleration );
                  }

                  //Add ship acceleration vector to velocity vector
                  playerList[j].theShip.xVel += xVelPre * ratioMult;
                  playerList[j].theShip.yVel += yVelPre * ratioMult;

                  //Add ship velocity vector to position vector
                  playerList[j].theShip.x += playerList[j].theShip.xVel * speedMod * ratioMult;
                  playerList[j].theShip.y += playerList[j].theShip.yVel * speedMod * ratioMult;

// Show the number of milliseconds between frames, and the resultant ratio used to modify speed
/*
if ( ( SDL_GetTicks() % 25 ) == 0 )
{
                  myStringstream.clear();
                  myStringstream.str("");
                  myStringstream << "Ticks: " << SDL_GetTicks() - updateTimer2 << " Speed Multiplier: " << ( SDL_GetTicks() - updateTimer2 ) / ( 1000 / 30.0 ) ;


                  SDL_WM_SetCaption( myStringstream.str().c_str(), NULL );
}
*/


                  if ( fireABullet )
                  {
                        playerList[j].theShip.createStandardBullet();
                        playerList[j].theShip.bulletTime = SDL_GetTicks();
                  }
            }
            //If ship is dead
            else
            {
                  if ( ( SDL_GetTicks() - playerList[j].theShip.timeOfDeath ) * speedMod > playerList[j].theShip.timeToRegen * 1000 )
                  {
                        playerList[j].theShip.invincible = true;
                        playerList[j].theShip.justRegenerated = true;
                        playerList[j].theShip.exist = true;
                        playerList[j].theShip.timeOfRegen = SDL_GetTicks();
                  }
            }
      }
}

void particleListUpdate()
{
      int i, j,
          cameFromShip,
          particleListSize = particleList.size(),
          playerListSize = playerList.size();

      //For each particle in the particle list
      for ( i = 0 ; i < particleListSize ; i++ )
      {
            //Check if particle is dead
            if ( ( SDL_GetTicks() - particleList[i].startTime ) * speedMod > particleList[i].maxLife )
            {
                  particleList[i].exist = 0;
            }
            else
            {
                  cameFromShip = false;

                  for ( j = 0 ; j < playerListSize ; j++ )
                        if ( particleList[i].ID == playerList[j].theShip.ID )
                              cameFromShip = true;

                  //If particle comes from a ship, randomize image frame indicator
                  if ( cameFromShip )
                        particleList[i].frameInd = rand() % NUM_OF_IMAGE_FRAMES;

                  //Update position of particles
                  particleList[i].x += particleList[i].xVel * speedMod * ratioMult;
                  particleList[i].y += particleList[i].yVel * speedMod * ratioMult;
            }
      }
}

void turretListUpdate()
{
      int i,
          closestPlayer,
          turretListSize = turretList.size();

      float distToShip,
            newDistance,
            initialProjectedDistance,
            distanceToBeMoved,
            projX, projY,
            initialProjX, initialProjY,
            initialShipX, initialShipY,
            distanceMoved,
            shipX, shipY,
            addDist, minusDist,
            addX, addY, minusX, minusY,
            percentDistanceToMove;

      //For each turret in the turret list, update if ship exists
      for ( i = 0 ; i < turretListSize ; i++ )
      {
            closestPlayer = turretList[i].playerWithClosestShip;

            if (  closestPlayer == 0 || playerList[closestPlayer].theShip.exist == false || SDL_GetTicks() - turretList[i].timeSinceTargeting > timeBetweenTurretCalibrations)
            {
                  turretList[i].findClosestShip();
                  closestPlayer = turretList[i].playerWithClosestShip;
                  turretList[i].timeSinceTargeting = SDL_GetTicks();
            }

            if ( playerList[closestPlayer].theShip.exist )
            {
                  distToShip = distance( turretList[i].x, turretList[i].y, playerList[closestPlayer].theShip.x, playerList[closestPlayer].theShip.y );
                  initialShipX = playerList[closestPlayer].theShip.x - turretList[i].x;
                  initialShipY = playerList[closestPlayer].theShip.y - turretList[i].y;
                  shipX = initialShipX / distToShip;
                  shipY = initialShipY / distToShip;

                  addX = cos( turretList[i].angle + 5 );
                  addY = sin( turretList[i].angle + 5 );
                  minusX = cos( turretList[i].angle - 5 );
                  minusY = sin( turretList[i].angle - 5 );

                  addDist = distance( addX, addY,
                                      shipX, shipY );
                  minusDist = distance( minusX, minusY,
                                        shipX, shipY );

                  initialProjX = distToShip * cos( turretList[i].angle );
                  initialProjY = distToShip * sin( turretList[i].angle );

                  initialProjectedDistance = distance( initialProjX, initialProjY,
                                                       initialShipX, initialShipY );


                  if ( addDist <= minusDist )
                  {
                        projX = distToShip * cos( turretList[i].angle - turretList[i].rotateSpeed );
                        projY = distToShip * sin( turretList[i].angle - turretList[i].rotateSpeed );

                        distanceMoved = distance( initialProjX, initialProjY,
                                                  projX, projY );

                        distanceToBeMoved = distance( initialProjX, initialProjY,
                                                      initialShipX, initialShipY );

                        percentDistanceToMove = distanceToBeMoved / distanceMoved;

                        if ( percentDistanceToMove > 1 )
                              percentDistanceToMove = 1;

                        turretList[i].angle -= turretList[i].rotateSpeed * percentDistanceToMove;

                  }
                  else if ( addDist > minusDist )
                  {
                        projX = distToShip * cos( turretList[i].angle + turretList[i].rotateSpeed );
                        projY = distToShip * sin( turretList[i].angle + turretList[i].rotateSpeed );

                        distanceMoved = distance( initialProjX, initialProjY,
                                                  projX, projY );

                        distanceToBeMoved = distance( initialProjX, initialProjY,
                                                      initialShipX, initialShipY );

                        percentDistanceToMove = distanceToBeMoved / distanceMoved;

                        if ( percentDistanceToMove > 1 )
                              percentDistanceToMove = 1;

                        turretList[i].angle += turretList[i].rotateSpeed *  percentDistanceToMove * speedMod * ratioMult;
                  }

                  projX = distToShip * cos( turretList[i].angle );
                  projY = distToShip * sin( turretList[i].angle );

                  turretList[i].projX = projX;
                  turretList[i].projY = projY;

                  newDistance = distance( projX, projY,
                                          initialShipX, initialShipY );

                  //Create new bullet if enough time has elapsed and if facing the ship
                  if( ( SDL_GetTicks() - turretList[i].bulletTime ) * speedMod  * ( realFPS / FRAMES_PER_SECOND ) > turretList[i].shotRechargeTime && newDistance < 30 )
                  {
                        turretList[i].createStandardBullet();
                        turretList[i].bulletTime = SDL_GetTicks();
                  }
            }
      }
}

void checkBounds( int checkShips )
{
      int i,
          bulletListSize = bulletList.size(),
          objectListSize = objectList.size(),
          particleListSize = particleList.size(),
          playerListSize = playerList.size();

      //Check bounds for bullets
      for ( i = 0 ; i < bulletListSize ; i++ )
      {
            if ( bulletList[i].x > MAX_SCREEN_WIDTH + bulletList[i].image->w / 2 )
                  bulletList[i].x -= MAX_SCREEN_WIDTH + bulletList[i].image->w;
            if ( bulletList[i].x < 0 - bulletList[i].image->w / 2 )
                  bulletList[i].x += MAX_SCREEN_WIDTH + bulletList[i].image->w;

            if ( bulletList[i].y > MAX_SCREEN_HEIGHT + bulletList[i].image->h / 2 )
                  bulletList[i].y -= MAX_SCREEN_HEIGHT + bulletList[i].image->h;
            if ( bulletList[i].y < 0 - bulletList[i].image->h / 2 )
                  bulletList[i].y += MAX_SCREEN_HEIGHT + bulletList[i].image->h;
      }

      //Check bounds for objects
      for ( i = 0 ; i < objectListSize ; i++ )
      {
            if ( objectList[i].x > MAX_SCREEN_WIDTH + objectList[i].image->w / 2 )
                  objectList[i].x -= MAX_SCREEN_WIDTH + objectList[i].image->w;
            if ( objectList[i].x < 0 - objectList[i].image->w / 2 )
                  objectList[i].x += MAX_SCREEN_WIDTH + objectList[i].image->w;

            if ( objectList[i].y > MAX_SCREEN_HEIGHT + objectList[i].image->h / 2 )
                  objectList[i].y -= MAX_SCREEN_HEIGHT + objectList[i].image->h;
            if ( objectList[i].y < 0 - objectList[i].image->h / 2 )
                  objectList[i].y += MAX_SCREEN_HEIGHT + objectList[i].image->h;
      }

      //Check bounds for particles
      for ( i = 0 ; i < particleListSize ; i++ )
      {
            if ( particleList[i].x > MAX_SCREEN_WIDTH + particleList[i].image->w / 2 )
                  particleList[i].x -= MAX_SCREEN_WIDTH + particleList[i].image->w;
            if ( particleList[i].x < 0 - particleList[i].image->w / 2 )
                  particleList[i].x += MAX_SCREEN_WIDTH + particleList[i].image->w;

            if ( particleList[i].y > MAX_SCREEN_HEIGHT + particleList[i].image->h / 2 )
                  particleList[i].y -= MAX_SCREEN_HEIGHT + particleList[i].image->h;
            if ( particleList[i].y < 0 - particleList[i].image->h / 2 )
                  particleList[i].y += MAX_SCREEN_HEIGHT + particleList[i].image->h;
      }

      if ( checkShips )
      {
            //Check bounds for all ships
            for ( i = 0 ; i < playerListSize ; i++ )
            {
                  if ( playerList[i].theShip.x > MAX_SCREEN_WIDTH + shipImages[ playerList[i].theShip.colorNum ].image->w / 2 )
                        playerList[i].theShip.x -= MAX_SCREEN_WIDTH + shipImages[ playerList[i].theShip.colorNum ].image->w;
                  if ( playerList[i].theShip.x < 0 - shipImages[ playerList[i].theShip.colorNum ].image->w / 2 )
                        playerList[i].theShip.x += MAX_SCREEN_WIDTH + shipImages[ playerList[i].theShip.colorNum ].image->w;

                  if ( playerList[i].theShip.y > MAX_SCREEN_HEIGHT + shipImages[ playerList[i].theShip.colorNum ].image->h / 2 )
                        playerList[i].theShip.y -= MAX_SCREEN_HEIGHT + shipImages[ playerList[i].theShip.colorNum ].image->h;
                  if ( playerList[i].theShip.y < 0 - shipImages[ playerList[i].theShip.colorNum ].image->h / 2 )
                        playerList[i].theShip.y += MAX_SCREEN_HEIGHT + shipImages[ playerList[i].theShip.colorNum ].image->h;
            }
      }
}

//Destroy the ship
void destroyShip( int playerNum )
{
      createExplosion( ( shipImages[ playerList[playerNum].theShip.colorNum ].image->w * shipImages[ playerList[playerNum].theShip.colorNum ].image->h ) / 6,
                         5000,
                         playerList[playerNum].theShip.x,
                         playerList[playerNum].theShip.y,
                         playerList[playerNum].theShip.xVel,
                         playerList[playerNum].theShip.yVel,
                         shipImages[playerList[playerNum].theShip.colorNum].particleImage );

      myStringstream.clear();
      myStringstream.str("");
      myStringstream << "Player " << playerNum + 1 << " died and has " << playerList[playerNum].lives - 1 << " lives remaining";

      addEvent( myStringstream.str().c_str(), 3000 );

      playerList[playerNum].theShip.exist = 0;
      playerList[playerNum].theShip.timeOfDeath = SDL_GetTicks();
      playerList[playerNum].theShip.xVel = 0;
      playerList[playerNum].theShip.yVel = 0;
      playerList[playerNum].theShip.x = rand() % SCREEN_WIDTH;
      playerList[playerNum].theShip.y = rand() % SCREEN_HEIGHT;

      playerList[playerNum].lives--;
}

void eventNoticeUpdate()
{
      int i, xyz = 0, xyz2 = 0,
          offset = 0,
          eventNoticeListSize = eventNoticeList.size();

      for ( i = 0 ; i < eventNoticeListSize ; i++ )
      {
            if ( ( SDL_GetTicks() - eventNoticeList[i].startTime ) > eventNoticeList[i].maxLife )
                  eventNoticeList[i].exist = false;
      }

      //Remove dead event notices, this is separate from removeAllDead so that old events are more likely to be removed
      for ( i = 0 ; i < eventNoticeListSize ; i++ )
            if ( !eventNoticeList[i].exist)
            {
                  xyz = eventNoticeList[i].notice->h;
                  xyz2 = eventNoticeList.size();

                  if ( eventNoticeList[i-offset].notice != NULL ) SDL_FreeSurface( eventNoticeList[i-offset].notice );
                  eventNoticeList.erase( eventNoticeList.begin() + i - offset );
                  offset++;
            }
}

void updateRealFPS()
{
      realFPS = (int)( frame / ( ( SDL_GetTicks() - updateTimer ) / 1000.0 ) );

      //Restart the update timer
      updateTimer = SDL_GetTicks();
      frame = 0;
}
