//Returns total number of objects (including +1 for the ship)
int getNumOfObjects()
{
      return objectList.size() + bulletList.size() + particleList.size() + 1;
}

//Returns total number of asteroids
int getNumOfAsteroids()
{
      int i,
          numOfAsteroids = 0,
          objectListSize = objectList.size();

      for ( i = 0 ; i < objectListSize ; i++ )
            numOfAsteroids += ( objectList[i].name == ASTEROID );

      return numOfAsteroids;
}

void createExplosion(int numOfParticles, float maxSpeed, float x, float y, float initXVel, float initYVel, SDL_Surface *myImage)
{
      int i;

      float angle,
            speed;

      particle myParticle;
      myParticle.x = x;
      myParticle.y = y;
      myParticle.image = myImage;

      for (i = 0 ; i < numOfParticles ; i++ )
      {
            angle = ( rand() % 36000 ) / 100.0;
            speed = ( rand() % ( (int)maxSpeed ) ) / 600.0 + pow( ( rand() % (int)( maxSpeed / 30.0 ) ) / 100.0 , 5 );

            myParticle.xVel = sin( angle ) * speed + initXVel;
            myParticle.yVel = cos( angle ) * speed + initYVel;
            myParticle.maxLife = 1000 + ( rand() % 10000 ) / 10;
            myParticle.frameInd = rand() % NUM_OF_IMAGE_FRAMES;
            particleList.push_back( myParticle );
      }
}

//Play music if not playing. Parameters are changeMusic, which is bool that indicates if change in music is being forced, and randomMusicFile, which is optional forcing of which song to pick
void checkMusic( int changeMusic, int randomMusicFile, int addMusicEvent = true )
{
      if ( randomMusicFile == 0 )
            randomMusicFile = rand() % 6 + 1;

      //If not playing music
      if ( ( !Mix_PlayingMusic() || changeMusic ) && playMusic )
      {
            Mix_FreeMusic(music);

            music = NULL;

            switch( randomMusicFile )
            {
                  case 1:
                        music = Mix_LoadMUS( "data/music/01.ogg" );
                        if ( music != NULL && addMusicEvent )
                              addEvent( "Song changed to track 1", 3000 );
                        break;
                  case 2:
                        music = Mix_LoadMUS( "data/music/02.ogg" );
                        if ( music != NULL && addMusicEvent )
                              addEvent( "Song changed to track 2", 3000 );
                        break;
                  case 3:
                        music = Mix_LoadMUS( "data/music/03.ogg" );
                        if ( music != NULL && addMusicEvent )
                              addEvent( "Song changed to track 3", 3000 );
                        break;
                  case 4:
                        music = Mix_LoadMUS( "data/music/04.ogg" );
                        if ( music != NULL && addMusicEvent )
                              addEvent( "Song changed to track 4", 3000 );
                        break;
                  case 5:
                        music = Mix_LoadMUS( "data/music/05.ogg" );
                        if ( music != NULL && addMusicEvent )
                              addEvent( "Song changed to track 5", 3000 );
                        break;
                  case 6:
                        music = Mix_LoadMUS( "data/music/06.ogg" );
                        if ( music != NULL && addMusicEvent )
                              addEvent( "Song changed to track 6", 3000 );
                        break;
                  case 7:
                        music = Mix_LoadMUS( "data/music/title.ogg" );
                        if ( music != NULL && addMusicEvent )
                              addEvent( "Song changed to title track", 3000 );
                        break;
                  default:
                        music = Mix_LoadMUS( "data/music/01.ogg" );
                        if ( music != NULL && addMusicEvent )
                              addEvent( "Song changed to track 1", 3000 );
                        break;
            }

            Mix_PlayMusic( music, 0 );
      }
      //If music playing when it shouldn't be
      else if ( Mix_PlayingMusic() && !playMusic )
            Mix_HaltMusic();
}

void checkFullscreenToggle()
{
      //Check if alt-enter combination is being pressed (to toggle fullscreen on or off)
      if ( ( keystates[ SDLK_RALT ] || keystates[ SDLK_LALT ] ) && keystates[ SDLK_RETURN ] )
      {
            fullscreen = !fullscreen;

            //Set up the screen
            if ( fullscreen )
            {
                  createScreen( true, SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP );
                  addEvent( "Changed to fullscreen mode", 3000 );
            }
            else
            {
                  createScreen( true, SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP );
                  addEvent( "Changed to windowed mode", 3000 );
            }
      }
}

int checkFirstKeyPushed()
{
      keystates = SDL_GetKeyState( NULL );
      return 0;
}

void displayTitle()
{
      text = TTF_RenderText_Solid( largeFont, "SDL ASTEROIDS", grayColour );
      applySurface( SCREEN_WIDTH / 2 - text->w/2 + 4, text->h / 3 + 4, text, screen );
      SDL_FreeSurface( text );
      text = TTF_RenderText_Solid( largeFont, "SDL ASTEROIDS", blackColour );
      applySurface( SCREEN_WIDTH / 2 - text->w/2, text->h / 3, text, screen );
      SDL_FreeSurface( text );
}
