//The (sort of) universal class to represent an object in space
class spaceObject
{
      public:

      //Name of object (type: asteroid, bullet, etc.)
      int name;

      //Unique id for this object
      int ID;

      //Does object still exist?
      int exist;

      //Location in space
      double x, y;

      //The velocity of the object
      double xVel,
             yVel;

      //Whether or not object has a gravitational pull
      int hasPull;

      //Whether or not object can move
      int canMove;

      //Whether or not object is affected by gravity
      int canBePulled;

      //Whether object can be affected by bullets
      int affectedByBullets;

      //Speed at which object is propelled after being broken up (if it can be broken up) - 1 is pretty sedate, 3+ is awesome
      float breakupSpeed;

      //Whether object will neutralize bullets if touched
      int killsBullets;

      //Whether object is lethal if it touches ship
      int killsShips;

      //The mass of the object
      float mass;

      //Size of object (for asteroids) pertaining to image (usually same as numOfLives)
      int sizeOfObject;

      //How many lives object has left before destroyed
      int numOfLives;

      //Pointer to the images used for the object
      SDL_Surface *image;
      SDL_Surface *secondImage;

      //The constructor
      spaceObject();
};

spaceObject::spaceObject()
{
      //Initialize variables to standard and safe values
      name = NOTHING;
      exist = 1;
      x = SCREEN_WIDTH / 2.0;
      y = SCREEN_HEIGHT / 2.0;
      xVel = 0;
      yVel = 0;
      hasPull = false;
      canMove = false;
      canBePulled = true;
      affectedByBullets = false;
      breakupSpeed = 1;
      numOfLives = -1;
      killsBullets = false;
      killsShips = false;
      mass = 0;
      image = NULL;
      ID = globalID;
      globalID++;
}

//Create a Planet and return it
spaceObject createPlanet( float x, float y, float mass )
{
      spaceObject myObject;

      myObject.x = x;
      myObject.y = y;

      myObject.name = PLANET;
      myObject.hasPull = true;
      myObject.canMove = false;
      myObject.mass = mass;
      myObject.xVel = 0;
      myObject.yVel = 0;
      myObject.killsBullets = true;

      myObject.image = planetImage;

      return myObject;
}

//Create an asteroid and return it
spaceObject createAsteroid( double x, double y, float xVel, float yVel, int hasPull, int imageSize, int asteroidLives, float breakupSpeed )
{
      spaceObject myObject;

      myObject.x = x;
      myObject.y = y;

      myObject.name = ASTEROID;
      myObject.hasPull = hasPull;
      myObject.canMove = true;
      myObject.canBePulled = false;
      myObject.affectedByBullets = IS_BROKEN_UP;
      myObject.breakupSpeed = breakupSpeed;
      myObject.killsBullets = true;
      myObject.killsShips = true;
      myObject.numOfLives = asteroidLives;
      myObject.sizeOfObject = imageSize;
      myObject.mass = imageSize * 8.0 ;
      myObject.xVel = xVel;
      myObject.yVel = yVel;

      switch ( imageSize )
      {
            case 1:
                  myObject.image = asteroidImage_1;
                  break;
            case 2:
                  myObject.image = asteroidImage_2;
                  break;
            case 3:
                  myObject.image = asteroidImage_3;
                  break;
            case 4:
                  myObject.image = asteroidImage_4;
                  break;
            case 5:
                  myObject.image = asteroidImage_5;
                  break;
            default:
                  myObject.image = asteroidImage_1;
                  break;
      }

      return myObject;
}
