class ship
{
      public:

      //Unique id for this object
      int ID;

      //Does ship still exist? (rather, is it dead or not)
      int exist;

      //If dead, when was the ship killed? (in ticks)
      int timeOfDeath;

      //How long after dying can the ship be recreated? (in seconds)
      int timeToRegen;

      //Time that ship was recreated (in ticks)
      int timeOfRegen;

      //Was the ship just regenerated?
      int justRegenerated;

      //How long does the ship stay just generated? (in seconds)
      int lengthOfNewness;

      //The number of the color ship chosen
      int colorNum;

      //Location in space
      double x, y;

      //The velocity of the object
      double xVel,
             yVel;

      //Whether or not object has a gravitational pull
      int hasPull;

      //Whether or not object is affected by gravity
      int canBePulled;

      //Whether bullets are attracted to Asteroids
      int shootsMagneticBullets;

      //Keeps track of time since last bullet was shot (in ticks)
      int bulletTime;

      //Keeps track of time since last particle was expelled (in ticks)
      int particleTime;

      //How much time between bullet shots (in ticks)
      int shotRechargeTime;

      //How much time between particle emissions (in ticks)
      int particleRechargeTime;

      //Speed at which bullets are shot
      float shotSpeed;

      //The mass of the object
      float mass;

      //The angle of the object (from the vertical)
      double angle;

      //The thrust power of the object
      float thrust;

      //The Rotation speed of the object
      float rotateSpeed;

      //Is the ship invincible?
      int invincible;

      //Does the ship kill other ships with its bullets?
      int killsOtherShips;

      //Pointer to the image used for the object's gun
      SDL_Surface *secondImage;

      //The constructor
      ship();

      void createStandardBullet();

      void createParticleUp();
      void createParticleDown();
};

ship::ship()
{
      exist = true;
      colorNum = 0;
      secondImage = shipBulletImage;
      mass = .25;
      angle = 90;
      thrust = 0;
      canBePulled = true;
      justRegenerated = false;
      hasPull = false;
      invincible = false;
      ID = globalID;
      globalID++;
      particleRechargeTime = 34;
      particleTime = SDL_GetTicks() - particleRechargeTime;
      killsOtherShips = false;
      shootsMagneticBullets = false;
}

//Create a standard bullet and add it to the list of bullets
void ship::createStandardBullet()
{
      bullet myBullet;
      myBullet.x = this->x + ( shipImages[ this->colorNum ].image->w / ( 2.0 * scaleFactorX ) ) * cos( this->angle );
      myBullet.y = this->y + ( shipImages[ this->colorNum ].image->h / ( 2.0 * scaleFactorY ) ) * sin( this->angle );
      myBullet.xVel =  this->shotSpeed * ( cos( this->angle ) ) + this->xVel;
      myBullet.yVel =  this->shotSpeed * ( sin( this->angle ) ) + this->yVel;
      myBullet.hasPull = false;
      myBullet.canBePulled = false;
      myBullet.maxLife = 3500;
      myBullet.realTicksLeft = 3500;
      myBullet.ID = this->ID;
      myBullet.killsShips = this->killsOtherShips;
      myBullet.pulledByAsteroids = this->shootsMagneticBullets;

      bulletList.push_back( myBullet );
      if ( playSounds )
            Mix_PlayChannel( -1, shipFire, 0 );
}

//Create a particle and add it to the list of particles
void ship::createParticleUp()
{
      particle myParticle;
      myParticle.ID = this->ID;
      myParticle.x = this->x - ( shipImages[ this->colorNum ].image->w / ( 2.0 * scaleFactorX ) ) * cos( this->angle );
      myParticle.y = this->y - ( shipImages[ this->colorNum ].image->h / ( 2.0 * scaleFactorY ) ) * sin( this->angle );
      myParticle.xVel = -1 * ( PARTICLE_SPEED * ( cos( this->angle ) ) ) + ( rand() % 10000 - 5000) / 1500.0  + this->xVel;
      myParticle.yVel = -1 * ( PARTICLE_SPEED * ( sin( this->angle ) ) ) + ( rand() % 10000 - 5000) / 1500.0  + this->yVel;
      myParticle.maxLife = 800;
      myParticle.frameInd = rand() % NUM_OF_IMAGE_FRAMES;

      particleList.push_back( myParticle );
      this->particleTime = SDL_GetTicks();
}

void ship::createParticleDown()
{
      particle myParticle;
      myParticle.ID = this->ID;
      myParticle.x = this->x + ( shipImages[ this->colorNum ].image->w / ( 2.0 * scaleFactorX ) ) * cos( this->angle );
      myParticle.y = this->y + ( shipImages[ this->colorNum ].image->h / ( 2.0 * scaleFactorY ) ) * sin( this->angle );
      myParticle.xVel = ( PARTICLE_SPEED * cos( this->angle ) ) + ( rand() % 10000 - 5000) / 1500.0 + this->xVel;
      myParticle.yVel = ( PARTICLE_SPEED * sin( this->angle ) ) + ( rand() % 10000 - 5000) / 1500.0 + this->yVel;
      myParticle.maxLife = 800;
      myParticle.frameInd = rand() % NUM_OF_IMAGE_FRAMES;

      particleList.push_back( myParticle );
      this->particleTime = SDL_GetTicks();
}
